﻿UIParentLoadAddOn("Blizzard_PetJournal"); -- force petjournal to load ASAP

local _, MNT4 = ...

_G['MNT4'] = MNT4
MNT4.eventframe = CreateFrame('frame')
MNT4.eventframe:SetScript("OnEvent", function(self, event, ...) if MNT4[event] then return MNT4[event](MNT4, event, ...) end end)
function MNT4:RegisterEvent(...) return MNT4.eventframe:RegisterEvent(...) end
function MNT4:UnregisterEvent(...) return MNT4.eventframe:UnregisterEvent(...) end

MNT4:RegisterEvent("ADDON_LOADED")
MNT4:RegisterEvent("ADDON_LOADED")

local libMnts = LibStub("LibMounts-1.0")

MNT4.libMnts = libMnts

function MNT4:ADDON_LOADED(event, addon)
	self:UnregisterEvent("ADDON_LOADED")
	self.ADDON_LOADED = nil
	if IsLoggedIn() then self:PLAYER_LOGIN() else self:RegisterEvent("PLAYER_LOGIN") end
end

function MNT4:PLAYER_LOGIN(event, addon)

	if GetMacroIndexByName("Mounted") == 0 then
		CreateMacro("Mounted", 1, '')
	end

	local defaults = {
		['profile'] = {
			['macros'] = {
				["TwoMounts"] = "Two Mount Macro",
				["OneMount"] = "One Mount Macro",
				["NoMounts"] = "No Mount Macro",
			},
			['mounts'] = {
				['casttype'] = {}, -- store all our mounts in their category indicating cast or no cast
				['types'] = {}, -- store all mounts here in their type (value is group or true)
				['groups'] = {}, -- store all mount groups here
			},
			['mountsets'] ={ -- stores our different priorites here (default, group, instance, pvp, zone(s)) <-- different priories are prioritized in that order
				['default'] = {}, -- store our mount priorities and groups here
			},
			['mountsetpriorities'] = {
				[1] = 'default', -- default is always lowest (1) priority
			},
			['startMinimized'] = false
		},
		['global'] = {
			["macros"] = {
				["TwoMounts"] = {
					["Two Mount Macro"] = [[#showtooltip
/cast [nomounted, nomodifier] <primary>; [nomounted]<secondary>
/dismount [mounted]
/stopmacro
/cast [nomodifier] <primary>; <secondary>]],
				},
				["OneMount"] = {
					["One Mount Macro"] = [[#showtooltip
/cast [nomounted] <primary>
/dismount [mounted]
/stopmacro
/cast <primary>]],
				},
				["NoMounts"] = {
					["No Mount Macro"] = [[/script print('No mount possibilities found')]],
				},
			},
		},
	}
	self.db = LibStub("AceDB-3.0"):New("MNT4_SavedVariables", defaults, true)
	self.db.RegisterCallback(self, "OnProfileChanged", "UpdateProfile")
	self.db.RegisterCallback(self, "OnProfileCopied", "UpdateProfile")
	self.db.RegisterCallback(self, "OnProfileReset", "UpdateProfile")

	-- set our default state
	self.state = 'default'
	self.currentMountTable = self.db.profile.mountsets['default']

	-- Register with LibMounts
	libMnts.RegisterCallback(self, "MOUNT_TYPE_UPDATE")
	self:MOUNT_TYPE_UPDATE(nil, libMnts:GetCurrentMountType())

	-- we learned a new mount (lets update)
	self:RegisterEvent('COMPANION_LEARNED')
	--run our update routine at least once
	self:COMPANION_LEARNED()


	self:RegisterEvent('UNIT_SPELLCAST_START')
	--self:RegisterEvent('UNIT_SPELLCAST_SUCCEEDED')

	self:RegisterEvent("PLAYER_REGEN_DISABLED")
	self:RegisterEvent("PLAYER_REGEN_ENABLED")

	-- /sigh 3 events to get zone data
	self:RegisterEvent("ZONE_CHANGED")
	self:RegisterEvent("ZONE_CHANGED_INDOORS")
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA")
	self:ZONE_CHANGED() -- run once on startup

	-- update pvp status
	self:RegisterEvent("UNIT_FACTION")
	self:UNIT_FACTION() -- run once on startup

	-- update instance status
	self:RegisterEvent("PLAYER_ENTERING_WORLD")
	self:PLAYER_ENTERING_WORLD() -- run once on startup

	-- update group status
	self:RegisterEvent("GROUP_ROSTER_UPDATE")
	self:GROUP_ROSTER_UPDATE() -- run once on startup

	self:UnregisterEvent("PLAYER_LOGIN")
	self.PLAYER_LOGIN = nil


	-- Test Crap
	--MNT4:UpdateScriptPriorityFrame('state')
	--MNT4:UpdateMountPriorityFrame('ground')
	--MNT4.MainFrame:Show()
end

function MNT4:UpdateProfile()
	print('Update Profile')
	-- reset blacklists
	wipe(MNT4.wr_blacklist)
	wipe(MNT4.wr_reverseblacklist)
	-- scan all mounts to make sure our mount list is uptodate
	MNT4:COMPANION_LEARNED()
	-- make sure everything is uptodate :)
	for MountType, TypeTable in pairs(self.db.profile.mounts.types) do
		for Set in pairs(self.db.profile.mountsets) do
			for Mount, GroupName in pairs(TypeTable) do
				if GroupName == true then
					self.db.profile.mountsets[Set][MountType][Mount] = self.db.profile.mountsets[Set][MountType][Mount] or 50
				else
					self.db.profile.mountsets[Set][MountType][Mount] = nil
					self.db.profile.mountsets[Set][MountType][GroupName] = self.db.profile.mountsets[Set][MountType][GroupName] or 50
				end
			end
		end
	end
end


MNT4.state_vals = {}
MNT4.nonZoneKeys = {
	["pvp"] = true,
	["instance"] = true,
	["group"] = true,
	["default"] = true,
}
function MNT4:UpdateState()
	local t = self.db.profile.mountsetpriorities
	local state
	for i=#t, 2, -1 do
		if MNT4.nonZoneKeys[t[i]] then
			if self.state_vals[t[i]] then
				state = t[i]
			end
		else
			if t[i] == self.state_vals.inZone then
				state = self.state_vals.inZone
			end
		end
		if state then
			break
		end
	end
	if not state then
		state = 'default'
	end
	if state ~= self.state  then
		self.state = state
		--print("---<<<UpdateState>>>---")
		--print(state)
		self.currentMountTable = self.db.profile.mountsets[state]
		self:UpdateMacro()
	end
end

function MNT4:GROUP_ROSTER_UPDATE()
	local v = UnitInRaid("unit") or GetNumSubgroupMembers() > 0
	if v ~= self.state_vals.group then
		self.state_vals.group = v
		self:UpdateState()
	end
end
MNT4.RAID_ROSTER_UPDATE = MNT4.GROUP_ROSTER_UPDATE

function MNT4:PLAYER_ENTERING_WORLD()
	local v = IsInInstance()
	if v ~= self.state_vals.instance then
		self.state_vals.instance = v
		self:UpdateState()
	end
end

function MNT4:UNIT_FACTION(event, unit)
	if unit == 'player' then
		local v = UnitIsPVPFreeForAll("player") or UnitIsPVP("player")
		if v ~= self.state_vals.pvp then
			self.state_vals.pvp = v
			self:UpdateState()
		end
	end
end

function MNT4:ZONE_CHANGED()
	local v = GetRealZoneText()
	if v ~= self.state_vals.inZone then
		self.state_vals.inZone = v
		self:UpdateState()
	end
end
MNT4.ZONE_CHANGED_INDOORS = MNT4.ZONE_CHANGED
MNT4.ZONE_CHANGED_NEW_AREA = MNT4.ZONE_CHANGED

---------------------------------------------------
------ Check database and add Unknown Mounts ------
---------------------------------------------------

MNT4.unsumonable = {} -- add all mounts that we have but can not summon (proffession mounts mainly) pure flying mounts are already excluded because no area is flyable if you cant :P
MNT4.sumonable = {} -- all mounts that we have and can summon need to check this so we do not call upon other faction mounts
MNT4.spellIDtoIndex = {} -- use this for the interface
MNT4.longestMountName = '' -- used for macro length check

function MNT4:COMPANION_LEARNED()
	local newMounts = 0
	local mTypes = {}
	wipe(MNT4.unsumonable) -- clear and repopulate
	wipe(MNT4.sumonable) -- clear and repopulate
	wipe(MNT4.spellIDtoIndex) -- clear and repopulate
	for id = 1,GetNumCompanions("MOUNT") do
		local _, _, mountSpellID = GetCompanionInfo("MOUNT", id)
		local mntName = GetSpellInfo(mountSpellID)
		if mntName:len() > MNT4.longestMountName:len() then
			MNT4.longestMountName = mntName
		end
		MNT4.spellIDtoIndex[mountSpellID] = id
		if libMnts:GetProfessionRestriction(mountSpellID) then
			MNT4.sumonable[mountSpellID] = true
		else
			MNT4.unsumonable[mountSpellID] = true
		end
		local ground, air, water, speed, location = libMnts:GetMountInfo(mountSpellID)

		if location then
			mTypes[#mTypes+1] = location
		else
			if not speed then
				if ground then
					mTypes[#mTypes+1] = 'ground'
				end
				if air then
					mTypes[#mTypes+1] = 'air'
				end
				if water then
					mTypes[#mTypes+1] = 'water'
				end
			end
		end
		for i, mType in ipairs(mTypes) do
			self.db.profile.mounts.types[mType] = self.db.profile.mounts.types[mType] or {}
			self.db.profile.mounts.groups[mType] = self.db.profile.mounts.groups[mType] or {}
			if self.db.profile.mounts.types[mType][mountSpellID] == nil then
				local _, _, _, _, _, _, castingTime = GetSpellInfo(mountSpellID)
				self.db.profile.mounts.casttype[mountSpellID] = castingTime > 0
				self.db.profile.mounts.types[mType][mountSpellID] = true
				for _, set in pairs(self.db.profile.mountsets) do
					set[mType] = set[mType] or {}
					set[mType][mountSpellID] = 50
				end
			end
		end
		wipe(mTypes)
	end

	-- lookup all spellnames for spell cast check
	self.MountCastTypeCast = {}
	self.MountCastTypeInstant = {}
	for id, castType in pairs(self.db.profile.mounts.casttype) do
		if castType then
			self.MountCastTypeCast[GetSpellInfo(id)] = true
		else
			self.MountCastTypeInstant[GetSpellInfo(id)] = true
		end
	end
	self:UpdateMacro()
end

------------------------------------------
------------ Update our Macro ------------
------------------------------------------

function MNT4:PLAYER_REGEN_DISABLED()
	self:UnregisterEvent('UNIT_SPELLCAST_START')
	self:UnregisterEvent('UNIT_SPELLCAST_SUCCEEDED')
end

function MNT4:PLAYER_REGEN_ENABLED()
	self:RegisterEvent('UNIT_SPELLCAST_START')
	self:RegisterEvent('UNIT_SPELLCAST_SUCCEEDED')
	libMnts:GetCurrentMountType()
end

function MNT4:UNIT_SPELLCAST_START(event, unit, spellName)
	if self.MountCastTypeCast[spellName] then
		self:UpdateMacro()
	end
end

function MNT4:UNIT_SPELLCAST_SUCCEEDED(event, unit, spellName)
	if self.MountCastTypeInstant[spellName] then
		self:UpdateMacro()
	end
end


MNT4.availablemounts = {}
function MNT4:MOUNT_TYPE_UPDATE(event, primary, secondary, tertiary)
	--print(event, primary, secondary)
	wipe(self.availablemounts)

	local mounts = self.db.profile.mounts.types
	if mounts[primary] and next(mounts[primary]) then
		self.availablemounts[#self.availablemounts+1] = primary
	end
	if mounts[secondary] and next(mounts[secondary]) then
		self.availablemounts[#self.availablemounts+1] = secondary
	end
	if mounts[tertiary] and next(mounts[tertiary]) then
		self.availablemounts[#self.availablemounts+1] = tertiary
	end

	MNT4:UpdateMacro()
end

function MNT4:UpdateMacro()
	--print("MNT4:UpdateMacro")
	if not InCombatLockdown() then
		local macro, err
		if self.availablemounts[2] then
			err, macro = self:ParseMacro(self.db.global.macros.TwoMounts[self.db.profile.macros.TwoMounts])
		elseif self.availablemounts[1] then
			err, macro = self:ParseMacro(self.db.global.macros.OneMount[self.db.profile.macros.OneMount])
		else
			err, macro = self:ParseMacro(self.db.global.macros.NoMounts[self.db.profile.macros.NoMounts])
		end
		EditMacro(GetMacroIndexByName("Mounted"), "Mounted", "INV_Misc_QuestionMark", macro)
	end
end


function MNT4TEST()
	for i=1,GetNumCompanions("MOUNT") do
		local creatureID, creatureName, creatureSpellID, icon, issummoned, mountType = GetCompanionInfo("MOUNT", i);
		if mountType == 29 or mountType == 31 or mountType == 12 then

		else
			print(creatureID, creatureSpellID, creatureName, icon, issummoned, mountType)
		end
	end
end
