local _, MNT4 = ...


MNT4.MountGroupingFrame = {}
function MNT4.MountGroupingFrame:Show()
	local frame = CreateFrame("frame", nil, UIParent)
		frame:SetPoint("CENTER")
		frame:SetWidth(300)
		frame:SetHeight(400)
		frame:Show()

	local typetitle = MNT4:GetWidget("title", frame)
		typetitle:SetPoint("TOPLEFT", frame, "TOPLEFT", 2, -2)
		typetitle:SetPoint("RIGHT", frame, "RIGHT", 0, 0)
		typetitle:SetText("Mount Groupings")
	frame.typetitle = typetitle

	local typedropdown = MNT4:GetWidget("dropdownbutton", frame)
		typedropdown:SetPoint("TOPLEFT", typetitle, "BOTTOMLEFT", 0,-4)
		typedropdown:SetPoint("RIGHT", typetitle, "CENTER", 0,-2)
		typedropdown:SetText("ground")
	frame.typedropdown = typedropdown

	local NEWbutton = MNT4:GetWidget("button", frame)
		NEWbutton:SetPoint("TOPLEFT", typedropdown, "TOPRIGHT", 0, 2)
		NEWbutton:SetPoint("RIGHT", typetitle, "RIGHT", 0, 0)
		NEWbutton:SetText(NEW.." "..GROUP)
	frame.NEWbutton = NEWbutton

	local scrollframe = MNT4:GetWidget("scrollframe", frame)
		scrollframe:SetPoint("TOPLEFT", typedropdown, "BOTTOMLEFT", 0,-2)
		scrollframe:SetPoint("RIGHT", NEWbutton, "RIGHT", 0, 0)
		scrollframe:SetPoint("BOTTOM", frame, "BOTTOM", 0,2)
	frame.scrollframe = scrollframe

	-- create our little Mount Type dropdown menu
	local MountTypeDropDownMenu = CreateFrame("Frame", "MountedMountTypeDropDownMenu")

	local function typeSelect(dropdownbutton, arg1)
		typedropdown:SetText(arg1)
		MNT4.MountGroupingFrameStateType = arg1
		MNT4:UpdateMountGroupingFrame()
	end

	local info = {}
	MountTypeDropDownMenu.initialize = function(self, level)
	    if not level then return end
		wipe(info)

		if level == 1 then
			local list = {}
			for t in pairs(MNT4.db.profile.mounts.types) do
				list[#list+1] = t
			end
			table.sort(list, function(a,b) return a > b end)

			for i, t in ipairs(list) do
				info.text = t
				info.arg1 = t
				info.checked = typedropdown:GetText() == t
				info.func = typeSelect
				UIDropDownMenu_AddButton(info, level)
			end
		end
	end

	typedropdown.dropButton:SetScript("OnClick", function(self, button, down)
		ToggleDropDownMenu(1, nil, MountTypeDropDownMenu, typedropdown, 0, 0)
	end)

	NEWbutton:SetScript("OnClick", function(self, button)
		if not MNT4.MountGroupEditorFrame:IsVisible() then
			 MNT4:OpenMountGroupEditor()
		end
	end)
	frame:SetScript("OnShow", function() MNT4:UpdateMountGroupingFrame() end)

	MNT4.MountGroupingFrame = frame
	MNT4.MountGroupingFrameStateSet = 'default' -- store which mountset we are looking at
	MNT4.MountGroupingFrameStateType = 'ground' -- store type of mounts we are looking at
	--MNT4:UpdateMountGroupingFrame()
end

local function grouplistbuttonClick(widget, button)
	MNT4:OpenMountGroupEditor(widget._mount)
end

local function setlistUP(widget, button)
	local position = widget:GetParent()._SetPos
	local name = widget:GetParent()._SetName
	table.remove(MNT4.db.profile.mountsetpriorities, position) -- remove from table
	table.insert(MNT4.db.profile.mountsetpriorities, position+1, name) -- add in new location
	MNT4:UpdateMountGroupingFrame()
	MNT4:UpdateState()
end

local function setlistDOWN(widget, button)
	local position = widget:GetParent()._SetPos
	local name = widget:GetParent()._SetName
	table.remove(MNT4.db.profile.mountsetpriorities, position) -- remove from table
	table.insert(MNT4.db.profile.mountsetpriorities, position-1, name) -- add in new location
	MNT4:UpdateMountGroupingFrame()
	MNT4:UpdateState()
end

local function setlistDEL(widget, button)
	local position = widget:GetParent()._SetPos
	local name = widget:GetParent()._SetName
	table.remove(MNT4.db.profile.mountsetpriorities, position) -- remove from table
	MNT4.db.profile.mountsets[name] = nil -- remove the mount set priorities
	MNT4:UpdateMountGroupingFrame()
	MNT4:UpdateState()
end

local mounts = {}
function MNT4:UpdateMountGroupingFrame()
	self.MountGroupingFrame:Show()
	local frame = self.MountGroupingFrame
	if not self.db.profile.mountsets[self.MountGroupingFrameStateSet] then
		self.MountGroupingFrameStateSet = 'default'
	end
	do -- type stuff
		frame.typedropdown:SetText(self.MountGroupingFrameStateType)

		--frame.scrollframe.scroll:SetVerticalScroll(0)
		frame.scrollframe.scrollbar:SetValue(0)
		local child = frame.scrollframe.child
		self.MountGroupingLines = self.MountGroupingLines or {}
		local lines = self.MountGroupingLines
		for i, w in pairs(lines) do
			if w.sep then
				w.sep = self:ReturnWidget(w.sep)
			end
			self:ReturnWidget(w)
			lines[i] = nil
		end

		wipe(mounts)
		for k, v in pairs(self.db.profile.mounts.types[self.MountGroupingFrameStateType]) do
			if v == true then
				mounts[k] = true
			else
				mounts[v] = 'grouped'
			end
		end
		local sortedmounts = {}
		for m in pairs(mounts) do
			sortedmounts[#sortedmounts+1] = m
		end
		table.sort(sortedmounts, function(a,b)
			if mounts[a] ~= 'grouped' then
				a = GetSpellInfo(a)
			end
			if mounts[b] ~= 'grouped' then
				b = GetSpellInfo(b)
			end
			return a:lower() < b:lower()
		end)
		for i, m in ipairs(sortedmounts) do
			local widget = self:GetWidget("MountGroupEdit", child)
			if next(lines) then
				widget.sep = self:GetWidget("HorizontalSeparator", child)
				widget.sep:SetPoint("TOPLEFT", lines[#lines], "BOTTOMLEFT", 0, 0)
				widget.sep:SetPoint("TOPRIGHT", lines[#lines], "BOTTOMRIGHT", 0, 0)
				widget.sep:Show()
				widget:SetPoint("TOPLEFT", widget.sep, "BOTTOMLEFT", 0, 0)
				widget:SetPoint("TOPRIGHT", widget.sep, "BOTTOMRIGHT", 0, 0)
			else
				widget:SetPoint("TOPLEFT", child, "TOPLEFT", 0, 0)
				widget:SetPoint("TOPRIGHT", child, "TOPRIGHT", 0, 0)
			end
			if mounts[m] == 'grouped' then
				widget._group = true
				widget._mount = m
				widget._MountID = MNT4.spellIDtoIndex[next(self.db.profile.mounts.groups[self.MountGroupingFrameStateType][m])]
				widget:SetText(ORANGE_FONT_COLOR_CODE..m)
				widget.edit:Enable()
				widget.edit._mount = m
				widget.edit:SetScript("OnClick", grouplistbuttonClick)
			else
				widget._group = false
				widget._mount = m
				widget._MountID = MNT4.spellIDtoIndex[m]
				widget:SetText(GetSpellInfo(m))
				widget.edit:Disable()
			end

			widget:Show()
			lines[#lines+1] =  widget
			child:SetHeight(#lines*widget:GetHeight())
		end
	end
end

