local _, addon = ...

local function Disable(frame)
	frame.up:Disable()
	frame.down:Disable()
	frame.del:Disable()
	frame:GetFontString():SetTextColor(.5,.5,.5)
end

local function Enable(frame)
	frame.up:Enable()
	frame.down:Enable()
	frame.del:Enable()
	frame:GetFontString():SetTextColor(1,1,1)
end

local function OnClick(frame)
	if frame._MountID then
		MountJournal_Select(frame._MountID)
	end
end

local function Slider_OnValueChanged(slider)
	local widget = slider.frame
	widget.prioritytext:SetText(slider:GetValue())
	if widget._mount then
		if widget._group then
			addon.db.profile.mountsets[addon.MountPriorityFrameStateSet][addon.MountPriorityFrameStateType][widget._mount]["_priority"] = slider:GetValue()
		else
			addon.db.profile.mountsets[addon.MountPriorityFrameStateSet][addon.MountPriorityFrameStateType][widget._mount] = slider:GetValue()
		end
	end
end

local function OnUpdate(slider)
	if not slider.initialset then
		slider:SetValue(slider:GetValue())
		slider:SetScript("OnUpdate", nil)
	end
	slider.initialset = nil
end

local function SetValue(slider, value)
	slider:_SetValue(value)
	slider.frame.prioritytext:SetText(value)
	
	if slider.initialset then
		slider:SetScript("OnUpdate", OnUpdate)
	end
end

local SliderBackdrop  = {
	bgFile = "Interface\\Buttons\\UI-SliderBar-Background",
	edgeFile = "Interface\\Buttons\\UI-SliderBar-Border",
	tile = true, tileSize = 8, edgeSize = 8,
	insets = { left = 3, right = 3, top = 6, bottom = 6 }
}

local function Constructor()
	local frame = CreateFrame("Button", nil, UIParent)
		frame:SetHeight(22)
		frame:SetHighlightTexture([[Interface\QuestFrame\UI-QuestLogTitleHighlight]] , "ADD")
		frame:GetHighlightTexture():SetVertexColor(1,1,0.2)
		frame:EnableMouse(true)

	local editButton = addon:GetWidget("button")
		editButton:SetParent(frame)
		editButton:SetText("Edit")
		editButton:SetWidth(40)
		editButton:SetNormalFontObject("ChatFontNormal")
		editButton:SetHighlightFontObject("ChatFontNormal")
		editButton:GetHighlightFontObject():SetTextColor(1,1,0.2)
		editButton:SetDisabledFontObject("ChatFontNormal")
		editButton:SetPoint("RIGHT", frame, "RIGHT", 0, 0)
	frame.edit = editButton
	
	local frametext = frame:CreateFontString(nil, "OVERLAY", "GameFontNormal")
		frametext:SetPoint("LEFT", frame, "LEFT", 2, 0)
		frametext:SetPoint("RIGHT", editButton, "LEFT", -2, 0)
		frametext:SetText("TEST TEST TEST TEST")
	frame:SetFontString(frametext)
	frame:SetNormalFontObject("GameFontNormal")
	frame:SetHighlightFontObject("GameFontHighlight")
	frame:RegisterForClicks("LeftButtonUp")
	
	frame:SetScript('OnDisable', Disable)
	frame:SetScript('OnEnable', Enable)
	frame:SetScript('OnClick', OnClick)
	return frame
end

local function Neutralizer(frame)
	frame:SetParent(UIParent)
	frame:ClearAllPoints()
	frame:SetText("")
	frame:SetScript("OnEnter", nil)
	frame:SetScript("OnLeave", nil)
	frame.edit:SetScript("OnClick", nil)
	frame.edit:Enable()
	frame:Hide()
end

addon:RegisterWidget("MountGroupEdit", Constructor, Neutralizer)
