local addonname, addon = ...

local backdrop = {
	bgFile = [[Interface\Tooltips\UI-Tooltip-Background]],
	edgeFile = [[Interface\Tooltips\UI-Tooltip-Border]], edgeSize = 16,
	insets = { left = 4, right = 3, top = 4, bottom = 3 }
}

local function ScrollframeOnMouseUp(frame)
	frame:GetScrollChild():SetFocus()
end
local function ScrollframeOnSizeChanged(frame, width, height)
	frame:GetScrollChild():SetWidth(width)
end

local function textAreaOnCursorChanged(editbox, x, y, width, height)
	local editboxheight = editbox:GetHeight()
	local parentheight = editbox.parent:GetHeight()
	if editboxheight > parentheight then
		local _, sMax = editbox.scrollbar:GetMinMaxValues()
		local sVal = editbox.scrollbar:GetValue()
		local startbloc = ((sVal)/sMax)*(editboxheight-parentheight)
		y = math.abs(y)
		if y < startbloc then
			editbox.scrollbar:SetValue(sVal-height)
		elseif y+height > startbloc+parentheight then
			editbox.scrollbar:SetValue(sVal+height)
		end
	end
end

local function GetText(box)
	return box.textArea:GetText()
end

local function SetText(box, t)
	t = t:gsub("	", "    ")
	return box.textArea:SetText(t)
end

local counter = 0
local function Constructor()
	counter = counter + 1

	local box = CreateFrame("frame", nil, UIParent)
		box:SetWidth(100)
		box:SetHeight(100)
		box:SetBackdrop(backdrop)
		box:SetBackdropColor(0, 0, 0)
		box:SetBackdropBorderColor(0.4, 0.4, 0.4)
	
	local scroll = CreateFrame("ScrollFrame", addonname.."Scroll"..counter, box, "UIPanelScrollFrameTemplate")
		scroll:SetPoint("TOPLEFT", box, 5, -5)
		scroll:SetPoint("BOTTOMRIGHT", box, -26, 4)
		scroll:EnableMouse(true)
	box.scroll = scroll
	box.scrollbar = _G[addonname.."Scroll"..counter.."ScrollBar"]
	
	local textArea = CreateFrame("EditBox", addonname.."ScrollText"..counter, scroll)
		textArea:SetAutoFocus(false)
		textArea:SetMultiLine(true)
		textArea:SetFontObject(GameFontHighlightSmall) --GameFontHighlightSmall)
		textArea:SetMaxLetters(99999)
		textArea:EnableMouse(true)
		textArea:SetScript("OnEscapePressed", textArea.ClearFocus)
		textArea:SetWidth(450)
		textArea:SetScript("OnCursorChanged", textAreaOnCursorChanged)
		textArea.parent = scroll
		textArea.scrollbar = box.scrollbar
	box.textArea = textArea
	
	scroll:SetScrollChild(textArea)
	scroll:SetScript("OnMouseUp", ScrollframeOnMouseUp)
	scroll:SetScript("OnSizeChanged", ScrollframeOnSizeChanged)
	
	box.GetText = GetText
	box.SetText = SetText
	
	return box
end

local function Neutralizer(box)
	box:SetParent(UIParent)
	box:SetText("")
	box.scrollbar:SetValue(0)
	box:Hide()
end

addon:RegisterWidget("scrollingeditbox", Constructor, Neutralizer)
